#ifndef __LINUX_KVM_PARA_H
#define __LINUX_KVM_PARA_H

#include <linux/errno.h>

/*
 * Guest OS interface for KVM paravirtualization
 *
 * Note: this interface is considered experimental and may change without
 *       notice.
 */

#define KVM_CR3_CACHE_SIZE	4

struct kvm_cr3_cache_entry {
	u64 guest_cr3;
	u64 host_cr3;
};

struct kvm_cr3_cache {
	u32 entry_count;
	u32 __pad;
	struct kvm_cr3_cache_entry entry[KVM_CR3_CACHE_SIZE];
} __attribute__ ((aligned(PAGE_SIZE)));

/*
 * Per-VCPU descriptor area shared between guest and host. Writable to
 * both guest and host. Registered with the host by the guest when
 * a guest acknowledges paravirtual mode.
 *
 * NOTE: all addresses are guest-physical addresses (gpa), to make it
 * easier for the hypervisor to map between the various addresses.
 */
struct kvm_vcpu_para_state {
	/*
	 * API version information for compatibility. If there's any support
	 * mismatch (too old host trying to execute too new guest) then
	 * the host will deny entry into paravirtual mode. Any other
	 * combination (new host + old guest and new host + new guest)
	 * is supposed to work - new host versions will support all old
	 * guest API versions.
	 */
	u32 guest_version;
	u32 host_version;
	u32 size;
	u32 ret;

	/*
	 * The address of the vm exit instruction (VMCALL or VMMCALL),
	 * which the host will patch according to the CPU model the
	 * VM runs on:
	 */
	u64 hypercall_gpa;

	/*
	 * Pointer to the struct kvm_cr3_cache CR3 cache:
	 */
	u64 cr3_cache_gpa;

} __attribute__ ((aligned(PAGE_SIZE)));

#define KVM_PARA_API_VERSION 1

/*
 * This is used for an RDMSR's ECX parameter to probe for a KVM host.
 * Hopefully no CPU vendor will use up this number. This is placed well
 * out of way of the typical space occupied by CPU vendors' MSR indices,
 * and we think (or at least hope) it wont be occupied in the future
 * either.
 */
#define MSR_KVM_API_MAGIC 0x87655678

#define KVM_EINVAL EINVAL

/*
 * Hypercall calling convention:
 *
 * Each hypercall may have 0-6 parameters.
 *
 * 64-bit hypercall index is in RAX, goes from 0 to __NR_hypercalls-1
 *
 * 64-bit parameters 1-6 are in the standard gcc x86_64 calling convention
 * order: RDI, RSI, RDX, RCX, R8, R9.
 *
 * 32-bit index is EAX, parameters are: EBX, ECX, EDX, ESI, EDI, EBP.
 * (the first 3 are according to the gcc regparm calling convention)
 *
 * No registers are clobbered by the hypercall, except that the
 * return value is in RAX.
 */
#define __NR_hypercall_load_cr3			0
#define __NR_hypercall_register_eth		1
#define __NR_hypercall_send_eth			2
#define __NR_hypercall_get_ktime		3
#define __NR_hypercalls				4

asmlinkage void __init kvm_probe(void);

#endif
